@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Spam Buttons Plugin
-  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
+  * Copyright (c) 2005-2009 Paul Lesniewski <paul@squirrelmail.org>,
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -19,28 +19,28 @@
 /* This function is used when not sending or adding attachments */
 function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $session='') {
     global $editor_size, $default_use_priority, $body, $idents,
-        $use_signature, $composesession, $data_dir, $username,
-        $username, $key, $imapServerAddress, $imapPort,
+        $use_signature, $data_dir, $username,
+        $username, $key, $imapServerAddress, $imapPort, $compose_messages,
         $composeMessage, $body_quote;
     global $languages, $squirrelmail_language, $default_charset;
-
-    /*
+                
+    /*      
      * Set $default_charset to correspond with the user's selection
      * of language interface. $default_charset global is not correct,
      * if message is composed in new window.
-     */
+     */             
     set_my_charset();
-
+                
     $send_to = $send_to_cc = $send_to_bcc = $subject = $identity = '';
     $mailprio = 3;
-
+            
     if ($passed_id) {
         $imapConnection = sqimap_login($username, $key, $imapServerAddress,
                 $imapPort, 0);
-
+            
         sqimap_mailbox_select($imapConnection, $mailbox);
         $message = sqimap_get_message($imapConnection, $passed_id, $mailbox);
-
+                
         $body = '';
         if ($passed_ent_id) {
             /* redefine the messsage in case of message/rfc822 */
@@ -48,13 +48,13 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
             /* message is an entity which contains the envelope and type0=message
              * and type1=rfc822. The actual entities are childs from
              * $message->entities[0]. That's where the encoding and is located
-             */
-
+             */ 
+            
             $entities = $message->entities[0]->findDisplayEntity
                 (array(), $alt_order = array('text/plain'));
             if (!count($entities)) {
                 $entities = $message->entities[0]->findDisplayEntity
-                    (array(), $alt_order = array('text/plain','text/html'));
+                    (array(), $alt_order = array('text/plain','html/plain'));
             }
             $orig_header = $message->rfc822_header; /* here is the envelope located */
             /* redefine the message for picking up the attachments */
@@ -63,33 +63,34 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
         } else {
             $entities = $message->findDisplayEntity (array(), $alt_order = array('text/plain'));
             if (!count($entities)) {
-                $entities = $message->findDisplayEntity (array(), $alt_order = array('text/plain','text/html'));
+                $entities = $message->findDisplayEntity (array(), $alt_order = array('text/plain', 'html/plain'));
             }
             $orig_header = $message->rfc822_header;
         }
 
-        $encoding = $message->header->encoding;
         $type0 = $message->type0;
         $type1 = $message->type1;
         foreach ($entities as $ent) {
+            $msg = $message->getEntity($ent);
+            $type0 = $msg->type0;
+            $type1 = $msg->type1;
             $unencoded_bodypart = mime_fetch_body($imapConnection, $passed_id, $ent);
             $body_part_entity = $message->getEntity($ent);
             $bodypart = decodeBody($unencoded_bodypart,
                     $body_part_entity->header->encoding);
             if ($type1 == 'html') {
                 $bodypart = str_replace("\n", ' ', $bodypart);
-                $bodypart = preg_replace(array('/<p>/i','/<br\s*(\/)*>/i'), "\n", $bodypart);
+                $bodypart = preg_replace(array('/<\/?p>/i','/<div><\/div>/i','/<br\s*(\/)*>/i','/<\/?div>/i'), "\n", $bodypart);
                 $bodypart = str_replace(array('&nbsp;','&gt;','&lt;'),array(' ','>','<'),$bodypart);
                 $bodypart = strip_tags($bodypart);
             }
             if (isset($languages[$squirrelmail_language]['XTRA_CODE']) &&
-                    function_exists($languages[$squirrelmail_language]['XTRA_CODE'])) {
+                    function_exists($languages[$squirrelmail_language]['XTRA_CODE'] . '_decode')) {
                 if (mb_detect_encoding($bodypart) != 'ASCII') {
-                    $bodypart = $languages[$squirrelmail_language]['XTRA_CODE']('decode', $bodypart);
+                    $bodypart = call_user_func($languages[$squirrelmail_language]['XTRA_CODE'] . '_decode', $bodypart);
                 }
             }
 
-            // charset encoding in compose form stuff
             if (isset($body_part_entity->header->parameters['charset'])) {
                 $actual = $body_part_entity->header->parameters['charset'];
             } else {
@@ -99,7 +100,6 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
             if ( $actual && is_conversion_safe($actual) && $actual != $default_charset){
                 $bodypart = charset_convert($actual,$bodypart,$default_charset,false);
             }
-            // end of charset encoding in compose
 
             $body .= $bodypart;
         }
@@ -111,6 +111,7 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
         } else {
             $mailprio = '';
         }
+        //ClearAttachments($session);
 
         $identity = '';
         $from_o = $orig_header->from;
@@ -128,8 +129,7 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
         $identities = array();
         if (count($idents) > 1) {
             foreach($idents as $nr=>$data) {
-                $enc_from_name = '"'.$data['full_name'].'" <'. $data['email_address'].'>';
-                if(strtolower($enc_from_name) == strtolower($orig_from)) {
+                if($enc_from_name == $orig_from) {
                     $identity = $nr;
                     break;
                 }
@@ -149,22 +149,25 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 $send_to = decodeHeader($orig_header->getAddr_s('to'),false,false,true);
                 $send_to_cc = decodeHeader($orig_header->getAddr_s('cc'),false,false,true);
                 $send_to_bcc = decodeHeader($orig_header->getAddr_s('bcc'),false,false,true);
-                // FIXME: ident support?
+                $send_from = $orig_header->getAddr_s('from');
+                $send_from_parts = new AddressStructure();
+                $send_from_parts = $orig_header->parseAddress($send_from);
+                $send_from_add = $send_from_parts->mailbox . '@' . $send_from_parts->host;
+                $identities = get_identities();
+                if (count($identities) > 0) {
+                    foreach($identities as $iddata) {
+                        if ($send_from_add == $iddata['email_address']) {
+                            $identity = $iddata['index'];
+                            break;
+                        }
+                    }
+                }
                 $subject = decodeHeader($orig_header->subject,false,false,true);
                 /* remember the references and in-reply-to headers in case of an reply */
                 $composeMessage->rfc822_header->more_headers['References'] = $orig_header->references;
                 $composeMessage->rfc822_header->more_headers['In-Reply-To'] = $orig_header->in_reply_to;
-                $body_ary = explode("\n", $body);
-                $cnt = count($body_ary) ;
-                $body = '';
-                for ($i=0; $i < $cnt; $i++) {
-                    if (!ereg("^[>\\s]*$", $body_ary[$i])  || !$body_ary[$i]) {
-                        sqWordWrap($body_ary[$i], $editor_size, $default_charset );
-                        $body .= $body_ary[$i] . "\n";
-                    }
-                    unset($body_ary[$i]);
-                }
-                sqUnWordWrap($body);
+                // rewrap the body to clean up quotations and line lengths
+                sqBodyWrap($body, $editor_size);
                 $composeMessage = getAttachments($message, $composeMessage, $passed_id, $entities, $imapConnection);
                 break;
             case ('edit_as_new'):
@@ -175,30 +178,24 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 $mailprio = $orig_header->priority;
                 $orig_from = '';
                 $composeMessage = getAttachments($message, $composeMessage, $passed_id, $entities, $imapConnection);
-                sqUnWordWrap($body);
+                // rewrap the body to clean up quotations and line lengths
+                sqBodyWrap($body, $editor_size);
                 break;
             case ('forward'):
                 $send_to = '';
-                $subject = decodeHeader($orig_header->subject,false,false,true);
-                if ((substr(strtolower($subject), 0, 4) != 'fwd:') &&
-                    (substr(strtolower($subject), 0, 5) != '[fwd:') &&
-                    (substr(strtolower($subject), 0, 6) != '[ fwd:')) {
-                    $subject = '[Fwd: ' . $subject . ']';
-                }
+                $subject = getforwardSubject(decodeHeader($orig_header->subject,false,false,true));
                 $body = getforwardHeader($orig_header) . $body;
+                // the logic for calling sqUnWordWrap here would be to allow the browser to wrap the lines
+                // forwarded message text should be as undisturbed as possible, so commenting out this call
+                // sqUnWordWrap($body);
                 $composeMessage = getAttachments($message, $composeMessage, $passed_id, $entities, $imapConnection);
+                //add a blank line after the forward headers
                 $body = "\n" . $body;
                 break;
             case ('forward_as_attachment'):
+                $subject = getforwardSubject(decodeHeader($orig_header->subject,false,false,true));
                 $composeMessage = getMessage_RFC822_Attachment($message, $composeMessage, $passed_id, $passed_ent_id, $imapConnection);
                 $body = '';
-/// CHANGE FOR SPAM_BUTTONS PLUGIN (added next 6 lines)
-                $subject = decodeHeader($orig_header->subject,false,false,true);
-                if ((substr(strtolower($subject), 0, 4) != 'fwd:') &&
-                    (substr(strtolower($subject), 0, 5) != '[fwd:') &&
-                    (substr(strtolower($subject), 0, 6) != '[ fwd:')) {
-                    $subject = '[Fwd: ' . $subject . ']';
-                }
                 break;
             case ('reply_all'):
                 if(isset($orig_header->mail_followup_to) && $orig_header->mail_followup_to) {
@@ -208,7 +205,8 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                     $send_to_cc = decodeHeader($send_to_cc,false,false,true);
                 }
             case ('reply'):
-                if (!$send_to) {
+                // skip this if send_to was already set right above here
+                if(!$send_to) {
                     $send_to = $orig_header->reply_to;
                     if (is_array($send_to) && count($send_to)) {
                         $send_to = $orig_header->getAddr_s('reply_to');
@@ -220,27 +218,31 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 }
                 $send_to = decodeHeader($send_to,false,false,true);
                 $subject = decodeHeader($orig_header->subject,false,false,true);
+                $subject = str_replace('"', "'", $subject);
                 $subject = trim($subject);
                 if (substr(strtolower($subject), 0, 3) != 're:') {
                     $subject = 'Re: ' . $subject;
                 }
                 /* this corrects some wrapping/quoting problems on replies */
                 $rewrap_body = explode("\n", $body);
-                $from = (is_array($orig_header->from) && !empty($orig_header->from)) ? $orig_header->from[0] : $orig_header->from;
-                sqUnWordWrap($body);
+                $from =  (is_array($orig_header->from)) ? $orig_header->from[0] : $orig_header->from;
                 $body = '';
-                $cnt = count($rewrap_body);
-                for ($i=0;$i<$cnt;$i++) {
-                    sqWordWrap($rewrap_body[$i], $editor_size, $default_charset);
-                    if (preg_match("/^(>+)/", $rewrap_body[$i], $matches)) {
+                $strip_sigs = getPref($data_dir, $username, 'strip_sigs');
+                foreach ($rewrap_body as $line) {
+                    if ($strip_sigs && substr($line,0,3) == '-- ') {
+                        break;
+                    }
+                    if (preg_match("/^(>+)/", $line, $matches)) {
                         $gt = $matches[1];
-                        $body .= $body_quote . str_replace("\n", "\n" . $body_quote
-                              . "$gt ", rtrim($rewrap_body[$i])) ."\n";
+                        $body .= $body_quote . str_replace("\n", "\n$body_quote$gt ", rtrim($line)) ."\n";
                     } else {
-                        $body .= $body_quote . (!empty($body_quote) ? ' ' : '') . str_replace("\n", "\n" . $body_quote . (!empty($body_quote) ? ' ' : ''), rtrim($rewrap_body[$i])) . "\n";
+                        $body .= $body_quote . (!empty($body_quote) ? ' ' : '') . str_replace("\n", "\n$body_quote" . (!empty($body_quote) ? ' ' : ''), rtrim($line)) . "\n";
                     }
-                    unset($rewrap_body[$i]);
                 }
+
+                //rewrap the body to clean up quotations and line lengths
+                $body = sqBodyWrap ($body, $editor_size);
+
                 $body = getReplyCitation($from , $orig_header->date) . $body;
                 $composeMessage->reply_rfc822_header = $orig_header;
 
@@ -248,7 +250,9 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
             default:
                 break;
         }
-/// CHANGE FOR SPAM_BUTTONS PLUGIN -- comment out next two lines
+/// CHANGE FOR SPAM_BUTTONS PLUGIN
+///        $compose_messages[$session] = $composeMessage;
+///        sqsession_register($compose_messages, 'compose_messages');
 ///        session_write_close();
 ///        sqimap_logout($imapConnection);
     }
@@ -263,53 +267,29 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
     return ($ret);
 } /* function newMail() */
 
-function getforwardHeader($orig_header) {
-    global $editor_size;
-
-    $display = array( _("Subject") => strlen(_("Subject")),
-            _("From")    => strlen(_("From")),
-            _("Date")    => strlen(_("Date")),
-            _("To")      => strlen(_("To")),
-            _("Cc")      => strlen(_("Cc")) );
-    $maxsize = max($display);
-    $indent = str_pad('',$maxsize+2);
-    foreach($display as $key => $val) {
-        $display[$key] = $key .': '. str_pad('', $maxsize - $val);
+function getforwardSubject($subject)
+{
+    if ((substr(strtolower($subject), 0, 4) != 'fwd:') &&
+            (substr(strtolower($subject), 0, 5) != '[fwd:') &&
+            (substr(strtolower($subject), 0, 6) != '[ fwd:')) {
+        $subject = '[Fwd: ' . $subject . ']';
     }
-    $from = decodeHeader($orig_header->getAddr_s('from',"\n$indent"),false,false,true);
-    $from = str_replace('&nbsp;',' ',$from);
-    $to = decodeHeader($orig_header->getAddr_s('to',"\n$indent"),false,false,true);
-    $to = str_replace('&nbsp;',' ',$to);
-    $subject = decodeHeader($orig_header->subject,false,false,true);
-    $subject = str_replace('&nbsp;',' ',$subject);
-    $bodyTop =  str_pad(' '._("Original Message").' ',$editor_size -2,'-',STR_PAD_BOTH) .
-        "\n". $display[_("Subject")] . $subject . "\n" .
-        $display[_("From")] . $from . "\n" .
-        $display[_("Date")] . getLongDateString( $orig_header->date, $orig_header->date_unparsed ). "\n" .
-        $display[_("To")] . $to . "\n";
-    if ($orig_header->cc != array() && $orig_header->cc !='') {
-        $cc = decodeHeader($orig_header->getAddr_s('cc',"\n$indent"),false,false,true);
-        $cc = str_replace('&nbsp;',' ',$cc);
-        $bodyTop .= $display[_("Cc")] .$cc . "\n";
-    }
-    $bodyTop .= str_pad('', $editor_size -2 , '-') .
-        "\n\n";
-    return $bodyTop;
+    return $subject;
 }
 
 function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
         $passed_ent_id='', $imapConnection) {
-    global $attachment_dir, $username, $data_dir, $uid_support;
+    global $attachment_dir, $username, $data_dir;
     $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
     if (!$passed_ent_id) {
         $body_a = sqimap_run_command($imapConnection,
                 'FETCH '.$passed_id.' BODY.PEEK[]',
                 TRUE, $response, $readmessage,
-                $uid_support);
+                TRUE);
     } else {
         $body_a = sqimap_run_command($imapConnection,
                 'FETCH '.$passed_id.' BODY.PEEK['.$passed_ent_id.']',
-                TRUE, $response, $readmessage, $uid_support);
+                TRUE, $response, $readmessage, TRUE);
         $message = $message->parent;
     }
     if ($response == 'OK') {
@@ -325,34 +305,23 @@ function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
         fwrite ($fp, $body);
         fclose($fp);
         $composeMessage->initAttachment('message/rfc822',$subject.'.msg',
-                $localfilename);
+                $full_localfilename);
     }
     return $composeMessage;
 }
 
 /**
  * temporary function to make use of the deliver class.
- * In the future the responsible backend should be automaticly loaded
+ * In the future the responsable backend should be automaticly loaded
  * and conf.pl should show a list of available backends.
  * The message also should be constructed by the message class.
- *
- * @param object $composeMessage The message being sent.  Please note
- *                               that it is passed by reference and
- *                               will be returned modified, with additional
- *                               headers, such as Message-ID, Date, In-Reply-To,
- *                               References, and so forth.
- *
- * @return boolean FALSE if delivery failed, or some non-FALSE value
- *                 upon success.
- *
  */
-function deliverMessage(&$composeMessage, $draft=false) {
+function deliverMessage($composeMessage, $draft=false) {
     global $send_to, $send_to_cc, $send_to_bcc, $mailprio, $subject, $body,
         $username, $popuser, $usernamedata, $identity, $idents, $data_dir,
         $request_mdn, $request_dr, $default_charset, $color, $useSendmail,
         $domain, $action, $default_move_to_sent, $move_to_sent;
     global $imapServerAddress, $imapPort, $sent_folder, $key;
-/// CHANGE FOR SPAM_BUTTONS PLUGIN
 /* --- do we need to do overrides again here?  should not need to, but
 //     someone reported problems with the default not being overriden
 //     when using this reporting method ---
@@ -432,12 +401,10 @@ function deliverMessage(&$composeMessage, $draft=false) {
     if (isset($request_mdn) && $request_mdn) {
         $rfc822_header->dnt = $rfc822_header->parseAddress($from_mail,true);
     }
-
     /* Receipt: On Delivery */
     if (isset($request_dr) && $request_dr) {
         $rfc822_header->more_headers['Return-Receipt-To'] = $from_mail;
     }
-
     /* multipart messages */
     if (count($composeMessage->entities)) {
         $message_body = new Message();
@@ -471,18 +438,10 @@ function deliverMessage(&$composeMessage, $draft=false) {
 
     $rfc822_header->content_type = $content_type;
     $composeMessage->rfc822_header = $rfc822_header;
-    if ($action == 'reply' || $action == 'reply_all') {
-        global $passed_id, $passed_ent_id;
-        $reply_id = $passed_id;
-        $reply_ent_id = $passed_ent_id;
-    } else {
-        $reply_id = '';
-        $reply_ent_id = '';
-    }
 
     /* Here you can modify the message structure just before we hand
        it over to deliver */
-/// CHANGE FOR SPAM_BUTTONS PLUGIN - comment out next 5 lines
+/// CHANGE FOR SPAM_BUTTONS PLUGIN
 ///    $hookReturn = do_hook('compose_send', $composeMessage);
 ///    /* Get any changes made by plugins to $composeMessage. */
 ///    if ( is_object($hookReturn[1]) ) {
@@ -495,75 +454,54 @@ function deliverMessage(&$composeMessage, $draft=false) {
         global $smtpServerAddress, $smtpPort, $pop_before_smtp;
 
         $authPop = (isset($pop_before_smtp) && $pop_before_smtp) ? true : false;
-
-        $user = '';
-        $pass = '';
-
         get_smtp_user($user, $pass);
-
         $stream = $deliver->initStream($composeMessage,$domain,0,
                 $smtpServerAddress, $smtpPort, $user, $pass, $authPop);
     } elseif (!$draft) {
         require_once(SM_PATH . 'class/deliver/Deliver_SendMail.class.php');
-        global $sendmail_path, $sendmail_args;
-        // Check for outdated configuration
-        if (!isset($sendmail_args)) {
-            if ($sendmail_path=='/var/qmail/bin/qmail-inject') {
-                $sendmail_args = '';
-            } else {
-                $sendmail_args = '-i -t';
-            }
-        }
-        $deliver = new Deliver_SendMail(array('sendmail_args'=>$sendmail_args));
+        global $sendmail_path;
+        $deliver = new Deliver_SendMail();
         $stream = $deliver->initStream($composeMessage,$sendmail_path);
     } elseif ($draft) {
         global $draft_folder;
+        require_once(SM_PATH . 'class/deliver/Deliver_IMAP.class.php');
         $imap_stream = sqimap_login($username, $key, $imapServerAddress,
                 $imapPort, 0);
         if (sqimap_mailbox_exists ($imap_stream, $draft_folder)) {
             require_once(SM_PATH . 'class/deliver/Deliver_IMAP.class.php');
             $imap_deliver = new Deliver_IMAP();
-            $succes = $imap_deliver->mail($composeMessage, $imap_stream, $reply_id, $reply_ent_id, $imap_stream, $draft_folder);
+            $length = $imap_deliver->mail($composeMessage);
+            sqimap_append ($imap_stream, $draft_folder, $length);
+            $imap_deliver->mail($composeMessage, $imap_stream);
+            sqimap_append_done ($imap_stream, $draft_folder);
             sqimap_logout($imap_stream);
             unset ($imap_deliver);
-            $composeMessage->purgeAttachments();
-            return $succes;
+            return $length;
         } else {
-            $msg  = '<br />'.sprintf(_("Error: Draft folder %s does not exist."),
-                htmlspecialchars($draft_folder));
+            $msg  = '<br />'.sprintf(_("Error: Draft folder %s does not exist."), $draft_folder);
             plain_error_message($msg, $color);
             return false;
         }
     }
     $succes = false;
     if ($stream) {
-        $deliver->mail($composeMessage, $stream, $reply_id, $reply_ent_id);
+        $length = $deliver->mail($composeMessage, $stream);
         $succes = $deliver->finalizeStream($stream);
     }
     if (!$succes) {
-        $msg  = _("Message not sent.") .' '.  _("Server replied:") .
-            "\n<blockquote>\n" . $deliver->dlv_msg . '<br />' .
-            $deliver->dlv_ret_nr . ' ' .
-            $deliver->dlv_server_msg . "</blockquote>\n\n";
+        $msg  = $deliver->dlv_msg . '<br />' .
+            _("Server replied: ") . $deliver->dlv_ret_nr . ' '.
+            $deliver->dlv_server_msg;
         plain_error_message($msg, $color);
     } else {
         unset ($deliver);
+        $move_to_sent = getPref($data_dir,$username,'move_to_sent');
         $imap_stream = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
 
-
-        // mark original message as having been replied to if applicable
-        global $passed_id, $mailbox, $action;
-        if ($action == 'reply' || $action == 'reply_all') {
-            sqimap_mailbox_select ($imap_stream, $mailbox);
-            sqimap_messages_flag ($imap_stream, $passed_id, $passed_id, 'Answered', false);
-        }
-
-
-        // copy message to sent folder
+        /* Move to sent code */
 /// CHANGE FOR SPAM_BUTTONS PLUGIN (added next 2 lines)
 global $sb_keep_copy_in_sent;
 if ($sb_keep_copy_in_sent) {
-        $move_to_sent = getPref($data_dir,$username,'move_to_sent');
         if (isset($default_move_to_sent) && ($default_move_to_sent != 0)) {
             $svr_allow_sent = true;
         } else {
@@ -584,19 +522,48 @@ if ($sb_keep_copy_in_sent) {
         }
 
         if (($fld_sent && $svr_allow_sent && !$lcl_allow_sent) || ($fld_sent && $lcl_allow_sent)) {
+            global $passed_id, $mailbox, $action;
+            if ($action == 'reply' || $action == 'reply_all') {
+                $save_reply_with_orig=getPref($data_dir,$username,'save_reply_with_orig');
+                if ($save_reply_with_orig) {
+                    $sent_folder = $mailbox;
+                }
+            }
+            sqimap_append ($imap_stream, $sent_folder, $length);
             require_once(SM_PATH . 'class/deliver/Deliver_IMAP.class.php');
             $imap_deliver = new Deliver_IMAP();
-            $imap_deliver->mail($composeMessage, $imap_stream, $reply_id, $reply_ent_id, $imap_stream, $sent_folder);
+            $imap_deliver->mail($composeMessage, $imap_stream);
+            sqimap_append_done ($imap_stream, $sent_folder);
             unset ($imap_deliver);
         }
 /// CHANGE FOR SPAM_BUTTONS PLUGIN (added next line)
 }
-        $composeMessage->purgeAttachments();
+        global $passed_id, $mailbox, $action;
+        ClearAttachments($composeMessage);
+        if ($action == 'reply' || $action == 'reply_all') {
+            sqimap_mailbox_select ($imap_stream, $mailbox);
+            sqimap_messages_flag ($imap_stream, $passed_id, $passed_id, 'Answered', false);
+        }
         sqimap_logout($imap_stream);
     }
     return $succes;
 }
 
+if (!function_exists('ClearAttachments'))
+{
+function ClearAttachments($composeMessage) {
+    if ($composeMessage->att_local_name) {
+        $attached_file = $composeMessage->att_local_name;
+        if (file_exists($attached_file)) {
+            unlink($attached_file);
+        }
+    }
+    for ($i=0, $entCount=count($composeMessage->entities);$i< $entCount; ++$i) {
+        ClearAttachments($composeMessage->entities[$i]);
+    }
+}
+}
+
 
 
 if (!function_exists('is_conversion_safe'))
@@ -611,77 +578,77 @@ if (!function_exists('is_conversion_safe'))
  * @return bool is it possible to convert to user's charset
  */
 function is_conversion_safe($input_charset) {
-    global $languages, $sm_notAlias, $default_charset, $lossy_encoding;
+  global $languages, $sm_notAlias, $default_charset, $lossy_encoding;
 
     if (isset($lossy_encoding) && $lossy_encoding )
         return true;
 
-    // convert to lower case
-    $input_charset = strtolower($input_charset);
+ // convert to lower case
+ $input_charset = strtolower($input_charset);
 
-    // Is user's locale Unicode based ?
-    if ( $default_charset == "utf-8" ) {
-        return true;
-    }
+ // Is user's locale Unicode based ?
+ if ( $default_charset == "utf-8" ) {
+   return true;
+ }
 
-    // Charsets that are similar
-    switch ($default_charset) {
-    case "windows-1251":
-        if ( $input_charset == "iso-8859-5" ||
+ // Charsets that are similar
+switch ($default_charset):
+case "windows-1251":
+      if ( $input_charset == "iso-8859-5" ||
            $input_charset == "koi8-r" ||
            $input_charset == "koi8-u" ) {
         return true;
-        } else {
-            return false;
-        }
-    case "windows-1257":
-        if ( $input_charset == "iso-8859-13" ||
-             $input_charset == "iso-8859-4" ) {
-            return true;
-        } else {
-            return false;
-        }
-    case "iso-8859-4":
-        if ( $input_charset == "iso-8859-13" ||
-             $input_charset == "windows-1257" ) {
-            return true;
-        } else {
-            return false;
-        }
-    case "iso-8859-5":
-        if ( $input_charset == "windows-1251" ||
-             $input_charset == "koi8-r" ||
-             $input_charset == "koi8-u" ) {
-            return true;
-        } else {
-            return false;
-        }
-    case "iso-8859-13":
-        if ( $input_charset == "iso-8859-4" ||
-             $input_charset == "windows-1257" ) {
-            return true;
-        } else {
-            return false;
-        }
-    case "koi8-r":
-        if ( $input_charset == "windows-1251" ||
-             $input_charset == "iso-8859-5" ||
-             $input_charset == "koi8-u" ) {
-            return true;
-        } else {
-            return false;
-        }
-    case "koi8-u":
-        if ( $input_charset == "windows-1251" ||
-             $input_charset == "iso-8859-5" ||
-             $input_charset == "koi8-r" ) {
-            return true;
-        } else {
-            return false;
-        }
-    default:
+     } else {
         return false;
-    }
+     }
+case "windows-1257":
+  if ( $input_charset == "iso-8859-13" ||
+       $input_charset == "iso-8859-4" ) {
+    return true;
+  } else {
+    return false;
+  }
+case "iso-8859-4":
+  if ( $input_charset == "iso-8859-13" ||
+       $input_charset == "windows-1257" ) {
+     return true;
+  } else {
+     return false;
+  }
+case "iso-8859-5":
+  if ( $input_charset == "windows-1251" ||
+       $input_charset == "koi8-r" ||
+       $input_charset == "koi8-u" ) {
+     return true;
+  } else {
+     return false;
+  }
+case "iso-8859-13":
+  if ( $input_charset == "iso-8859-4" ||
+       $input_charset == "windows-1257" ) {
+     return true;
+  } else {
+     return false;
+  }
+case "koi8-r":
+  if ( $input_charset == "windows-1251" ||
+       $input_charset == "iso-8859-5" ||
+       $input_charset == "koi8-u" ) {
+     return true;
+  } else {
+     return false;
+  }
+case "koi8-u":
+  if ( $input_charset == "windows-1251" ||
+       $input_charset == "iso-8859-5" ||
+       $input_charset == "koi8-r" ) {
+     return true;
+  } else {
+     return false;
+  }
+default:
+   return false;
+endswitch;
 }
 }
 
