@@ -98,8 +98,8 @@ function sb_button_action_do($args)
    sqGetGlobalVar('passed_ent_id',   $passed_ent_id,  SQ_FORM);
    sqGetGlobalVar('REQUEST_METHOD',  $method,         SQ_SERVER);
 
-   if (sqGetGlobalVar('location', $location, SQ_FORM))
-      $location = htmlspecialchars($location);
+   if (sqGetGlobalVar('location', $location, SQ_POST))
+      { /* $location = htmlspecialchars($location); */ }
    else
       $location = php_self();
 
